#import "MyPDFView.h"
#import "MyPDFDisplayView.h"
#import "MyPDFMatteView.h"
#import "MyPDFViewInternal.h"
#import "MyPDFViewPrivate.h"
#import <objc/runtime.h>





@class PDFMatteView, PDFDisplayView, PDFViewController, PDFPageTransition, PDFCoachMarkController, PDFPopupView ;







@interface PDFPrintView : NSView
{
    PDFDocument *_document;
    NSString *_printJobTitle;
    int _pageIndex;
    BOOL _autoRotate;
    BOOL _rotateRight;
    int _scaleMode;
}

- (void)dealloc;
- (void)setDocument:(PDFDocument*)document;
- (id)document;
- (void)setPrintJobTitle:(id)arg1;
- (id)printJobTitle;
- (BOOL)knowsPageRange:(struct _NSRange *)arg1;
- (struct _NSRect)rectForPage:(int)arg1;
- (void)drawRect:(struct _NSRect)arg1;

@end

@interface PDFPopupLayoutInfo : NSObject
{
    PDFPopupView *popupView;
    struct _NSRect bounds;
    struct _NSRect parentBounds;
}

- (float)maxY;
- (void)setMaxY:(float)arg1;
- (float)minY;
- (void)setMinY:(float)arg1;
- (struct _NSRect)parentBounds;
- (void)setParentBounds:(struct _NSRect)arg1;
- (struct _NSRect)bounds;
- (void)setBounds:(struct _NSRect)arg1;
- (id)popupView;
- (void)setPopupView:(id)arg1;

@end




static const float scrollBy = 10;


@implementation MyPDFView

- (id)initWithCoder:(NSCoder *)decoder
{
	self = [super initWithCoder:decoder];

	if (self != nil) {
		[self commonInit];
	}
	return self;
}

- (id)initWithFrame:(NSRect)frame {
	NSLog(@"foo");
	
	
	
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
		
		NSClipView *clip = [[NSClipView alloc] init];
		[clip release];
    }
	
	
	return self;
}

- (void)setNeedsDisplay:(BOOL)flag
{
	[super setNeedsDisplay:flag];
}



- (id)document
{	
	return _pdfPriv->pdfDoc;
}

- (void)setDocument:(PDFDocument*)doc 
{
	if (doc != _pdfPriv->pdfDoc) {
		[_pdfPriv->pdfDoc release];
		_pdfPriv->pdfDoc = [doc retain];
		
		[self resetDocumentState];
		
		NSNotificationCenter *dc = [NSNotificationCenter defaultCenter];
		
		[dc addObserver:self selector:@selector(documentWasUnlocked:) 
				   name:PDFDocumentDidUnlockNotification object:doc];
		
		
		[dc addObserver:self selector:@selector(documentAnnotationsChanged:)
				   name:PDFDocumentAnnotationsChangedNotification object:doc];
		
		[self layoutDocumentView];
		
		[dc postNotificationName:PDFViewChangedDocumentNotification object:self];
	}
}



- (void)goToFirstPage:(id)arg1
{
}


- (BOOL)canGoToLastPage
{
	BOOL r = NO;

	return r;
}


- (void)goToLastPage:(id)arg1
{
}


- (BOOL)canGoToNextPage
{
	BOOL r;
	return r;
}


- (void)goToNextPage:(id)arg1
{
}


- (BOOL)canGoToPreviousPage
{
	BOOL r = NO;
	return r;
}


- (void)goToPreviousPage:(id)arg1
{
}


- (BOOL)canGoBack
{
	BOOL r = NO;
	return r;
}


- (void)goBack:(id)arg1
{
}


- (BOOL)canGoForward
{
	BOOL r = NO;
	return r;
}


- (void)goForward:(id)arg1
{

}


- (PDFPage*)currentPage
{
	PDFPage *currentPage = nil;
	PDFDocument *doc = [self document];
	
	if (doc && [doc pageCount] > _pdfPriv->currentPageIndex) {
		currentPage = [doc pageAtIndex:_pdfPriv->currentPageIndex];
	}

	return currentPage;
}


- (void)goToPage:(id)arg1
{
}

- (PDFDestination*)currentDestination
{
	PDFDestination *dest = nil;

	if ([self document]) {
		PDFDisplayMode dispMode = [self functionalDisplayMode];
		if (dispMode == kPDFDisplaySinglePage || dispMode == kPDFDisplayTwoUp) {
			// single page
			PDFPage *page = [self currentPage];
			NSRect rect = [self normalizedPageBounds:page];
			NSPoint point = {0, rect.size.height};
			dest = [[PDFDestination alloc] initWithPage:page atPoint:point];
		} else {
			// continous scroll
			NSRect bounds = [self bounds];
			NSPoint point = {0, bounds.size.height};
			PDFPage *page = [self pageForPoint:point nearest:YES];
			point = [self convertPoint:point toPage:page];
			dest = [[PDFDestination alloc] initWithPage:page atPoint:point];
		}
	}
	
	return dest;
}


- (void)goToDestination:(PDFDestination*)destination
{
	if (destination != nil && 
		[destination page] != nil) {
		
		[self pushDestination:[self currentDestination]];
		
		if (_pdfPriv->scrollEntry == 0) {
			_pdfPriv->scrollEntry = 0x5;
			
			[self goToDestinationNoPush:destination];
			[self cancelScheduleSyncPageIndex];
		}
	}
	_pdfPriv->scrollEntry = 0;
}

- (float)PDFViewWillChangeScaleFactor:(id)sender toScale:(float)scale
{
	scale = MAX(scale, _pdfPriv->minScaleFactor);
	scale = MIN(scale, _pdfPriv->maxScaleFactor);
	return scale;
}


- (BOOL)performKeyEquivalent:(id)arg1
{
	BOOL p = [super performKeyEquivalent:arg1];
	return p;
}

- (void)keyDown:(NSEvent *)theEvent
{
	[super keyDown:theEvent];
}

- (void)scrollPageUp:(id)arg1
{
	[self scrollByPage:NO];
}

- (void)scrollPageDown:(id)arg1
{
	[self scrollByPage:YES];
}

- (void)scrollLineUp:(id)arg1
{
	[self scrollVerticalBy:-scrollBy];
}


- (void)scrollLineDown:(id)arg1
{
	[self scrollVerticalBy:scrollBy];
}

- (void)drawRect:(NSRect)dirtyRect
{
}


- (void)drawPage:(PDFPage*)page
{
	[page drawWithBox:[self displayBox]];
}

- (void)drawPagePost:(PDFPage*)page
{
}

- (id)pageForPoint:(NSPoint)point nearest:(BOOL)nearest
{
	PDFPage *page = nil;
	NSPoint convertedPoint = [self convertPoint:point toView:_pdfPriv->pagesView];
	page = [self nearestPageToPoint:convertedPoint];
	
	if (!nearest) {
		convertedPoint = [self convertPoint:point toPage:page];
		NSRect bounds = [page boundsForBox:[self displayBox]];
		if (!NSPointInRect(convertedPoint, bounds)) {
			page = nil;
		}
	}
	
	return page;
}



- (NSPoint)convertPoint:(NSPoint)point toPage:(PDFPage*)page
{
	point = [self convertPoint:point toView:_pdfPriv->pagesView];
	NSPoint origin = [self originForPageDisplay:page];
	float scale = [self scaleFactor];
	
	point.x /= scale;
	point.y /= scale;
	
	point.x -= origin.x;
	point.y -= origin.y;
	
	
	// TODO deal with rotated pages
	
	return point;
}



- (NSPoint)convertPoint:(NSPoint)point fromPage:(PDFPage*)page
{
	
	NSPoint origin = [self originForPageDisplay:page];
	float scale = [self scaleFactor];
	
	
	
	point.x += origin.x;
	point.y += origin.y;
	
	point.x *= scale;
	point.y *= scale;
	
	point = [self convertPoint:point fromView:_pdfPriv->pagesView];
	
	
	// TODO deal with rotated pages
	
	return point;
}

- (NSRect)convertRect:(NSRect)rect fromPage:(PDFPage *)page
{
	NSRect result;
	NSPoint tmp = {rect.origin.x + rect.size.width, rect.origin.y + rect.size.height};
	result.origin = [self convertPoint:rect.origin fromPage:page];
	tmp = [self convertPoint:tmp fromPage:page];
	result.size.height = tmp.y - result.origin.y;
	result.size.width = tmp.x - result.origin.x;
	return result;
}

- (NSRect) convertRect: (NSRect) rect toPage: (PDFPage *) page
{
	NSRect result;
	NSPoint tmp = {rect.origin.x + rect.size.width, rect.origin.y + rect.size.height};
	result.origin = [self convertPoint:rect.origin toPage:page];
	tmp = [self convertPoint:tmp toPage:page];
	result.size.height = tmp.y - result.origin.y;
	result.size.width = tmp.x - result.origin.x;
	return result;
}


- (void)layoutDocumentView
{

	NSNotificationCenter *dc = [NSNotificationCenter defaultCenter];
	
	if (_pdfPriv->pagesView != nil) {
		PDFDocument *doc = [self document];
		
		if (_pdfPriv->currentPageIndex >= [doc pageCount]) {
			_pdfPriv->currentPageIndex = [doc pageCount] - 1;
			
			[dc postNotificationName:PDFViewChangedPageNotification object:self];
		}
		
		[self freePageViewBounds];
		NSSize zero = {0,0};
		_pdfPriv->singlePageContSize = zero;
		_pdfPriv->twoUpContSize = zero;
		[self resizeDisplayView:self];
		PDFPage *cp = [self determineCurrentPage];
		if (_pdfPriv->currentPageIndex != [doc indexForPage:cp]) {
			[self goToPageNoPush:cp];
		}
		[dc postNotificationName:PDFViewDidLayoutDocumentViewNotification object: self];
	}
}



- (NSArray*)visiblePages
{
	
	NSMutableArray *array = nil;
	PDFDocument *doc = [self document];
	int pageCount;
	if (doc != nil && (pageCount = [doc pageCount]) > 0) {
		array = [[[NSMutableArray alloc] initWithCapacity:pageCount] autorelease];
		
		if (_pdfPriv->firstVisiblePage <= 0 || _pdfPriv->lastVisiblePage <= 0) {
			[self findVisiblePages];
		}
		
		for (int i = _pdfPriv->firstVisiblePage; i <= _pdfPriv->lastVisiblePage; i++) {
			[array addObject:[doc pageAtIndex:i-1]];
		}
	}
	
	return array;
}


-(void) periodicFlushRoutine:(id) sender
{
}

-(void) scrollWheel:(NSEvent *)theEvent
{
	NSLog(@"%s", __PRETTY_FUNCTION__);
	[super scrollWheel:(NSEvent *)theEvent];
}


- (void) setDisplayMode: (PDFDisplayMode) mode 
{
	if (_pdfPriv->displayMode != mode) {
		_pdfPriv->displayMode = mode;
		[self layoutDocumentView];
		_pdfPriv->scrollEntry = 0;
		
		if (mode == kPDFDisplaySinglePage ||
			mode == kPDFDisplayTwoUp) {
			[self reflectNewPageOn];
		}
		
		NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
		[defaultCenter postNotificationName:PDFViewDisplayModeChangedNotification object:self];  
	}
}

- (PDFDisplayMode) displayMode
{
	return _pdfPriv->displayMode;
}



-(BOOL) canGoToFirstPage
{	
	return _pdfPriv->currentPageIndex > 0;
}



-(BOOL) displaysPageBreaks 
{	
	return _pdfPriv->displaysPageBreaks;
}
	

-(int) displayBox
{	
	return _pdfPriv->displayBox;
}


-(BOOL) displaysAsBook
{	
	return _pdfPriv->displaysAsBook;
}


-(BOOL) shouldAntiAlias
{	
	return _pdfPriv->antiAliasing;
}


-(float) greekingThreshold
{	
	return _pdfPriv->greekingThreshold;
}


-(NSColor*) backgroundColor
{
	return _pdfPriv->matteColor;
}


-(id) delegate
{	
	return _pdfPriv->delegate;
}


-(float) scaleFactor
{
	return _pdfPriv->scaleFactor;
}


-(BOOL) autoScales
{	
	return _pdfPriv->autoScale;
}


-(BOOL) acceptsFirstResponder
{
	return YES;
}


-(BOOL) acceptsFirstMouse:(NSEvent *)theEvent
{
	return YES;
}


-(id) currentSelection
{	
	return _pdfPriv->currentSelection;
}


-(id) highlightedSelections
{	
	return _pdfPriv->highlights;
}


-(id) documentView
{	
	return _pdfPriv->pagesView;
}


-(BOOL) allowsDragging
{	
	return _pdfPriv->allowsDragging;
}


-(void) setEnableDataDetectors:(BOOL)enable
{
	_pdfPriv->doRunDataDetectors = enable;
}


-(BOOL) enableDataDetectors
{
	return _pdfPriv->doRunDataDetectors;
}


-(BOOL) prepareForDragOperation:(id <NSDraggingInfo>)sender
{
	return YES;
}

- (void)animation:(NSAnimation *)animation didReachProgressMark:(NSAnimationProgress)progress
{
}

- (float)animation:(NSAnimation *)animation valueForProgress:(NSAnimationProgress)progress
{
	return 0;
}

- (void)animationDidStop:(NSAnimation *)animation
{
}

- (BOOL)animationShouldStart:(NSAnimation *)animation
{
	return NO;
}

- (void) goToSelection: (PDFSelection *) selection
{
}


- (void) goToRect: (NSRect) rect onPage: (PDFPage *) page
{
}


- (void) setDisplaysPageBreaks: (BOOL) breaks
{
	if (_pdfPriv->displaysPageBreaks != breaks) {
		_pdfPriv->displaysPageBreaks = breaks;
		[self layoutDocumentView];
	}
}


- (void) setDisplayBox: (PDFDisplayBox) box
{
	// check if valid box
	if (box >= kPDFDisplayBoxMediaBox && box <= kPDFDisplayBoxArtBox) {
		if (box != [self displayBox]) {
			_pdfPriv->displayBox = box;
			[self layoutDocumentView];
			NSNotificationCenter *dc = [NSNotificationCenter defaultCenter];
			[dc postNotificationName:PDFViewDisplayBoxChangedNotification object:self];
		}
	} else {
		// TODO invalid box type
	}
}


- (void) setDisplaysAsBook: (BOOL) asBook
{
	if (_pdfPriv->displaysAsBook != asBook) {
		_pdfPriv->displaysAsBook = asBook;
		[self layoutDocumentView];
	}
}


- (void) setShouldAntiAlias: (BOOL) aliasing
{
}


- (void) setGreekingThreshold: (CGFloat) threshold
{
}

- (IBAction) takeBackgroundColorFrom: (id) sender
{
}


- (void) setBackgroundColor: (NSColor *) newColor
{
}

- (void) setDelegate: (id) anObject
{
}

- (void) setScaleFactor: (float) scale
{
	NSRect docRect = [_pdfPriv->scrollView documentVisibleRect];
	docRect.origin.x += docRect.size.width / 2.;
	docRect.origin.y += docRect.size.height / 2.;
	[self setScaleFactor:scale anchorPoint:docRect.origin];
}

- (IBAction) zoomIn: (id) sender
{
	[self setScaleFactor:[self scaleFactor] * _pdfPriv->zoomIncrement];
}


- (BOOL) canZoomIn
{
	BOOL zoom = NO;
	return zoom;
}


- (IBAction) zoomOut: (id) sender
{
	[self setScaleFactor:[self scaleFactor] / _pdfPriv->zoomIncrement];
}


- (BOOL) canZoomOut
{
	BOOL zoom = NO;
	return zoom;
}

- (void) setAutoScales: (BOOL) newAuto
{
	if (_pdfPriv->autoScale != newAuto) {
		_pdfPriv->autoScale = newAuto;
		[self layoutDocumentView];
	}
}

- (PDFAreaOfInterest) areaOfInterestForMouse: (NSEvent *) event
{
	PDFAreaOfInterest result = 0;
#if !NEW_METHODS
	result =[super areaOfInterestForMouse:event];
#endif
	return result;
}

- (void) setCursorForAreaOfInterest: (PDFAreaOfInterest) area
{
}

- (void) performAction: (PDFAction *) action
{
}


- (void) setCurrentSelection: (PDFSelection *) selection
{
}


- (void) setCurrentSelection: (PDFSelection *) selection animate: (BOOL) animate
{
}


- (void) clearSelection
{
}


- (IBAction) selectAll: (id) sender
{
}


- (void) scrollSelectionToVisible: (id) sender
{
}


- (void) setHighlightedSelections: (NSArray *) selections
{
}


- (void) takePasswordFrom: (id) sender
{
}

- (void) copy: (id) sender
{
}


- (void) printWithInfo: (NSPrintInfo *) printInfo autoRotate: (BOOL) doRotate
{
}


- (void) printWithInfo: (NSPrintInfo *) printInfo autoRotate: (BOOL) doRotate pageScaling: (PDFPrintScalingMode) scale
{
}


- (void) annotationsChangedOnPage: (PDFPage *) page
{
}


- (NSSize) rowSizeForPage: (PDFPage *) page
{
	NSSize size = [self viewSizeForPage:page];
	PDFDisplayMode mode = [self functionalDisplayMode];
	float scale = [self scaleFactor];
	
	if (mode == kPDFDisplaySinglePageContinuous || kPDFDisplayTwoUpContinuous) {
		size.height = [self pageViewHeight:page];
	} else {
		size.height *= scale;
	}
	
	size.width *= scale;
	
	size.height = roundf(size.height);
	size.width = roundf(size.width);
	
	return size;
}


- (void) setAllowsDragging: (BOOL) allow
{
}


- (void)magnifyWithEvent:(NSEvent *)event
{
	NSPoint point = [self convertPointFromBase:[event locationInWindow]];
	float magnify = 0;
	
	do {
		magnify += [event magnification];
		NSWindow *window = [self window];
		NSDate *date = [NSDate dateWithTimeIntervalSinceNow:0];
		unsigned mask = 0x40000000;
		NSString *mode = @"NSEventTrackingRunLoopMode";
		event = [window nextEventMatchingMask:mask untilDate:date inMode:mode dequeue:YES];
	} while (event != nil);
	 

	float scaleFactor = [self scaleFactor];
	
	scaleFactor = scaleFactor + scaleFactor * magnify;
	[self setScaleFactor:scaleFactor anchorPoint:point];
}


-(void) test:(id) sender
{
	SEL sel = @selector(scheduledRedisplay:);
	SEL s2 = @selector(documentWasUnlocked:);
	
	NSLog(@"0x%X, 0x%X", sel, s2);
	/*
	PDFDocument *doc = [self document];
	int count = [doc pageCount];
	
	for (int i = 0; i < count; i++) {
		PDFPage *page = [doc pageAtIndex:i];
		[page setRotation:0];
		NSPoint o = [super scrollOriginForPageTopLeft:page];
		NSPoint n = [self scrollOriginForPageTopLeft:page];
		NSLog(@"new: {%.3f, %.3f}, old: {%.3f,%.3f}", n.x, n.y, o.x, o.y);
	}
	 */
}

- (void)mouseDown:(NSEvent *)theEvent
{
	
	
	NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	NSRect rect = [_pdfPriv->scrollView documentVisibleRect];
	
	NSPoint p2 = [_pdfPriv->matteView convertPoint:p fromView:self];
	
	rect.origin.y += 1;
	
	[self constrainedScrollToPoint:rect.origin];
	
	//NSPoint trans = [self convertPoint:p toPage:[[self document] pageAtIndex:0]];
	
	//NSPoint ntrans = [self new_convertPoint:p toPage:[[self document] pageAtIndex:0]];
	
	//NSLog(@"convertPoint:\npoint:{%.3f,%.3f}, convert:{%.3f,%.3f}, new:{%.3f,%.3f}", p.x, p.y, trans.x, trans.y, ntrans.x, ntrans.y);
	
	[super mouseDown:theEvent];
}



@end
