#import "MyPDFViewInternal.h"
#import "MyPDFViewPrivate.h"

static const float PageViewPadding = 8.0;

#define PageBreakPad 8.

 NSString* PDFViewLastVisiblePage = @"PDFViewLastVisiblePage";
 NSString* PDFViewFirstVisiblePage = @"PDFViewFirstVisiblePage";
 NSString* PDFViewWasLastVisiblePage = @"PDFViewWasLastVisiblePage";
 NSString* PDFViewWasFirstVisiblePage = @"PDFViewWasFirstVisiblePage";
 NSString* PDFViewVisiblePagesChangedNotification = @"PDFViewVisiblePagesChanged";
 NSString* PDFViewChangedPageNotification = @"PDFViewChangedPage";
 NSString* PDFViewDidLayoutDocumentViewNotification = @"PDFViewDidLayoutDocumentView";
 NSString* PDFDocumentAnnotationsChangedNotification = @"PDFDocumentAnnotationsChanged";
 NSString* PDFViewChangedDocumentNotification = @"PDFViewChangedDocument";


@implementation  MyPDFView (MyPDFViewInternal)


- (void)commonInit
{
	_pdfPriv = [[PDFViewPrivateVars alloc] init];
	
	
	_pdfPriv->matteColor = [[NSColor colorWithDeviceRed:.5 green:.5 blue:.5 alpha:1.] retain];
	_pdfPriv->pageColor = [[NSColor whiteColor] retain];
	
	_pdfPriv->displayMode = kPDFDisplaySinglePageContinuous;
	_pdfPriv->displaysPageBreaks = YES;
	_pdfPriv->antiAliasing = YES;
	_pdfPriv->greekingThreshold = 3;
	_pdfPriv->lineWidthThreshold = 0.75;
	_pdfPriv->interpolation = YES;
	_pdfPriv->scaleFactor = 1.;
	_pdfPriv->maxScaleFactor = 20.;
	_pdfPriv->minScaleFactor = 0.1;
	//_pdfPriv->zoomIncrement = 1.18920004;
	_pdfPriv->zoomIncrement = 1.05;
	_pdfPriv->displayBox = 1;
	_pdfPriv->defaultGutterWidth = 224;
	_pdfPriv->flushInterval = 1;
	_pdfPriv->periodicFlush = YES;
	_pdfPriv->showTooltips = YES;
	_pdfPriv->destinationHistory = [[NSMutableArray alloc] initWithCapacity:10];
	_pdfPriv->animateFlag = 4;
	_pdfPriv->displaysMarkups = YES;
	_pdfPriv->initialBringUp = YES;
	_pdfPriv->trackingArea = [[NSTrackingArea alloc] initWithRect:NSMakeRect(0, 0, 0, 0) options:0x00000223 owner:self userInfo:nil];
	_pdfPriv->doRunDataDetectors = YES;
	
	[self initInternalViews];

}

- (BOOL)runningWithinInterfaceBuilder
{
	return NO;
}

- (void)loadSamplePDF
{
}

- (void)resetDocumentState
{	
}

- (PDFDisplayMode)functionalDisplayMode
{
	PDFDisplayMode displayMode = _pdfPriv->displayMode;
	PDFDocument *doc = [self document];
	
	if (doc) {
		int pc = [doc pageCount];
		if (pc == 0) {
			if (displayMode == kPDFDisplayTwoUp) {
				displayMode = kPDFDisplaySinglePage;
			}
			else if	(displayMode == kPDFDisplayTwoUpContinuous) {
				displayMode = kPDFDisplaySinglePageContinuous;
			}
		} else if (pc == 2 && displayMode == kPDFDisplayTwoUpContinuous) {
			displayMode = kPDFDisplaySinglePageContinuous;
		}
	}
	return displayMode;
}




/**
 * allocate the pageViewBounds variable, this is freed and nilled 
 * elsewhere by freePageViewBounds
 */
- (void)generatePageViewBounds
{
		
	if (_pdfPriv->pageViewBounds) {
		return;
	}
	
	const int lastPageIndex = [self lastPageIndex];
	const int pageCount = lastPageIndex + 1;
	const BOOL displaysPageBreaks = [self displaysPageBreaks];
	
	_pdfPriv->pageViewBounds = (NSRect*)NSAllocateCollectable(pageCount * sizeof(NSRect), 0);
	
	if (_pdfPriv->pageViewBounds == nil) {
		return;
	}
	
	PDFDisplayMode displayMode = [self functionalDisplayMode];
	
	switch (displayMode) {
		case kPDFDisplaySinglePage:
		{
			for (int index = 0; index < pageCount; index++) {
				PDFPage *page = [[self document] pageAtIndex:index];
				NSRect rect = [self normalizedPageBounds:page];
				
				if (displaysPageBreaks) {
					rect.origin.x += 4;
					rect.origin.y += 8;
				}
				
				_pdfPriv->pageViewBounds[index] = rect;
			}
		}
			break;
		case kPDFDisplaySinglePageContinuous:
		{
			float maxWidth = 0.;
			for (int index = lastPageIndex; index >= 0; index--) {
				PDFPage *page = [[self document] pageAtIndex:index];
				NSRect rect = [self normalizedPageBounds:page];
				
				if (displaysPageBreaks) {
					rect.origin.x += 4;
					rect.origin.y += 8;
				}
				
				if (index < lastPageIndex) {
					rect.origin.y += (_pdfPriv->pageViewBounds[index+1].origin.y +
									  _pdfPriv->pageViewBounds[index+1].size.height + 4);
				}
				
				maxWidth = MAX(maxWidth, rect.size.width);
				_pdfPriv->pageViewBounds[index] = rect;
			}
			
			for (int index = 0; index < pageCount; index++) {
				if (_pdfPriv->pageViewBounds[index].size.width < maxWidth) {
					_pdfPriv->pageViewBounds[index].origin.x +=
					round((maxWidth - _pdfPriv->pageViewBounds[index].size.width)/2.0);
				}
			}
		}
			break;
		case kPDFDisplayTwoUp:
		{
			for (int i = 0; i < pageCount; i++) {
				NSRect rect = [self normalizedPageBounds:[[self document] pageAtIndex:i]];
				
				if (displaysPageBreaks) {
					rect.origin.x += 4;
					rect.origin.y += 8;
				}
				
				if ((i+1)%2 == 0) {
					// is second page
					rect.origin.x += (_pdfPriv->pageViewBounds[i-1].origin.x +
									  _pdfPriv->pageViewBounds[i-1].size.width + 4);
					// center the smaller page
					if (_pdfPriv->pageViewBounds[i-1].size.height < rect.size.height) {
						_pdfPriv->pageViewBounds[i-1].origin.y += 
						round((rect.size.height - _pdfPriv->pageViewBounds[i-1].size.height) / 2.);
					}
					
					if (rect.size.height < _pdfPriv->pageViewBounds[i-1].size.height) {
						rect.origin.y += 
						round((_pdfPriv->pageViewBounds[i-1].size.height - rect.size.height) / 2.);
					}
				}
				
				_pdfPriv->pageViewBounds[i] = rect;
			}
		}
			break;
		case kPDFDisplayTwoUpContinuous:
		{
			float leftMaxWidth = 0.;
			float rightMaxWidth = 0.;
			float heights[(pageCount+1)/2];
			
			for (int i = 0; i < pageCount; i++) {
				NSRect rect = [self normalizedPageBounds:[[self document] pageAtIndex:i]];
				
				if (displaysPageBreaks) {
					rect.origin.x += 4;
					rect.origin.y += 8;
				}
				
				if ((i+1)%2 == 0) {
					// is right page
					rightMaxWidth = MAX(rightMaxWidth, rect.size.width);
					
					if (_pdfPriv->pageViewBounds[i-1].size.height < rect.size.height) {
						// right bigger than left
						heights[(i-1)/2] = rect.size.height + rect.origin.y;
						
						// center page vertically
						_pdfPriv->pageViewBounds[i-1].origin.y += 
						round((rect.size.height - _pdfPriv->pageViewBounds[i-1].size.height) / 2.);
					} 
					
					// center left page vertically
					if (rect.size.height < _pdfPriv->pageViewBounds[i-1].size.height) {
						rect.origin.y += 
						round((_pdfPriv->pageViewBounds[i-1].size.height - rect.size.height) / 2.);
					}
				} else {
					// is left page
					leftMaxWidth = MAX(leftMaxWidth, rect.size.width);
					heights[i/2] = rect.size.height + rect.origin.y;
				}
				
				_pdfPriv->pageViewBounds[i] = rect;
			}
			
			float height = 0.;
			
			for	(int i = lastPageIndex; i >= 0; i--) {
				_pdfPriv->pageViewBounds[i].origin.y += (height + 8);
				
				if ((i+1)%2 == 0) {
					// even page, (right column)
					_pdfPriv->pageViewBounds[i].origin.x += (leftMaxWidth + 8);
					
					if (_pdfPriv->pageViewBounds[i].size.width < rightMaxWidth) {
						_pdfPriv->pageViewBounds[i].origin.x +=
						round((rightMaxWidth - _pdfPriv->pageViewBounds[i].size.width)/2.0);
					}
					
				} else {
					if (_pdfPriv->pageViewBounds[i].size.width < leftMaxWidth) {
						_pdfPriv->pageViewBounds[i].origin.x +=
						round((leftMaxWidth - _pdfPriv->pageViewBounds[i].size.width)/2.0);
					}
					// odd page (left column), bump height for next iteration.
					height += heights[i/2];
				}
				
			}
		}
			break;
			
		default:
			break;
	}
	
	
	// 0x320fc0:	4	2420	612	792
	// 0x320fd0:	4	1616	612	792
	// 0x320fe0:	4	812		612	792
	// 0x320ff0:	4	8		612	792
	
	
	// two up, like pages
	// 0x15f280:	4	8	612	792
	// 0x15f290:	624	8	612	792
	// 0x15f2a0:	4	8	612	792
	// 0x15f2b0:	624	8	612	792
	
	// two up, contin
	// 0x3134d10:	4	812	612	792
	// 0x3134d20:	624	812	612	792
	// 0x3134d30:	4	8	612	792
	// 0x3134d40:	624	8	612	792
	
	// two up single page, unlike pages
	// 0x122790:	4	8	595	842
	// 0x1227a0:	607	60	522	738
	// 0x1227b0:	4	8	522	738
	// 0x1227c0:	534	8	522	738
	// 0x1227d0:	4	8	522	738
	// 0x1227e0:	534	8	522	738
	// 0x1227f0:	4	8	522	738
	
	// two page, cont
	// 0x435ea0:	4		2258	595	842
	// 0x435eb0:	643.5	2258	522	738
	// 0x435ec0:	40.5	1508	522	738
	// 0x435ed0:	643.5	1508	522	738
	// 0x435ee0:	40.5	758		522	738
	// 0x435ef0:	643.5	758		522	738
	// 0x435f00:	338		8		522	738
	
	// single page, cont
	// 0x132620:	4		4508	595	842
	// 0x132630:	40.5	3758	522	738
	// 0x132640:	40.5	3008	522	738
	// 0x132650:	40.5	2258	522	738
	// 0x132660:	40.5	1508	522	738
	// 0x132670:	40.5	758		522	738
	// 0x132680:	40.5	8		522	738

}



/**
 * free and nil-ify pageViewBounds,
 * called from:
 * reflectNewPageOn,
 * dealloc,
 * layoutDocumentView,
 */
- (void)freePageViewBounds
{
	
	if (_pdfPriv->pageViewBounds != nil) {
		_pdfPriv->pageViewBounds = nil;
	}
}



- (NSSize)viewSizeForPage:(PDFPage*)page
{
	NSSize result = {0,0};
		
	PDFDocument *document = [self document];
	const int index = [document indexForPage:page];
	const int pageCount = [document pageCount];
	
	if (_pdfPriv->pageViewBounds == nil) {
		[self generatePageViewBounds];
	}
	
	if (page && index >= 0) {
		BOOL displaysPageBreaks = [self displaysPageBreaks];
		switch ([self functionalDisplayMode]) {
			case kPDFDisplaySinglePage:
			{
				result = _pdfPriv->pageViewBounds[index].size;
				if (displaysPageBreaks) {
					result.width += PageViewPadding;
					result.height += PageViewPadding;
				}
			} break;
			case kPDFDisplaySinglePageContinuous:
			{
				result = _pdfPriv->pageViewBounds[0].size;
				result.height += _pdfPriv->pageViewBounds[0].origin.y;
				for(int i = 0; i < pageCount; i++) {
					result.width = MAX(result.width, _pdfPriv->pageViewBounds[i].size.width);
				}
				if (displaysPageBreaks) {
					result.width += PageViewPadding;
				}
			} break;
			case kPDFDisplayTwoUp:
			{
				PDFPage *facingPage = [self facingPageForPage:page];
				result = _pdfPriv->pageViewBounds[index].size;
				if (facingPage != nil) {
					NSSize facingSize = _pdfPriv->pageViewBounds[[document indexForPage:facingPage]].size;
					result.height = MAX(result.height, facingSize.height);
					result.width += facingSize.width;
					if (displaysPageBreaks) {
						result.width += PageViewPadding;
					}
				}
				if (displaysPageBreaks) {
					result.height += PageViewPadding;
					result.width += PageViewPadding;
				}
			} break;
			case kPDFDisplayTwoUpContinuous:
			{
				
				result = _pdfPriv->pageViewBounds[0].size;
				
				
				if ([self displaysAsBook] == NO && pageCount > 1) {
					result.height = MAX(result.height,  _pdfPriv->pageViewBounds[1].size.height);
					
				}
				
				result.height += _pdfPriv->pageViewBounds[0].origin.y;
				
				float left = 0;
				float right = 0;
				
				
				if ([self displaysAsBook]) {
					for(int i = 1; i < pageCount; i++) {
						if ((i % 2) != 0) {
							// even page -- left column
							left = MAX(left, _pdfPriv->pageViewBounds[i].size.width);
						} else  {
							right = MAX(right, _pdfPriv->pageViewBounds[i].size.width);
						}
					}
				} else {
					for(int i = 0; i < pageCount; i++) {
						if ((i % 2) == 0) {
							// even page -- left column
							left = MAX(left, _pdfPriv->pageViewBounds[i].size.width);
						} else  {
							right = MAX(right, _pdfPriv->pageViewBounds[i].size.width);
						}
					}
				}
				
				result.width = MAX(result.width, left + right);
				
				if (displaysPageBreaks) {
					result.width += 2*PageViewPadding;
				}
				
				
			} break;
			default:
				break;
		}
	}
	

	return result;
}



- (PDFPage*)nearestPageToPoint:(NSPoint)point
{
	PDFPage *result;
	
	PDFDisplayMode mode = [self functionalDisplayMode];
	PDFDocument *doc = [self document];
	int min = 0;
	int lastPageIndex = [self lastPageIndex];
	int max = lastPageIndex;
	float scale = [self scaleFactor];
	point.x /= scale;
	point.y /= scale;
	double padding = [self displaysPageBreaks] ? PageBreakPad / 2. : 0.;
	int mid = 0;
	
	switch (mode) {
		case kPDFDisplaySinglePage:
		{
			result = [doc pageAtIndex:_pdfPriv->currentPageIndex];
		} 
			break;
		case kPDFDisplayTwoUp:
		{
			// find x
			PDFPage *facingPage = [self facingPageForPage:[doc pageAtIndex:_pdfPriv->currentPageIndex]];
			int facingIndex = facingPage != nil ? [doc indexForPage:facingPage] : _pdfPriv->currentPageIndex;
			int left, right;
			if (_pdfPriv->pageViewBounds[_pdfPriv->currentPageIndex].origin.x < _pdfPriv->pageViewBounds[facingIndex].origin.x) {
				left = _pdfPriv->currentPageIndex;
				right = facingIndex;
			} else {
				left = facingIndex;
				right = _pdfPriv->currentPageIndex;
			}
			
			int index = point.x < _pdfPriv->pageViewBounds[right].origin.x - padding ? left : right;
			result = [doc pageAtIndex:index];
		}
			break;
		case kPDFDisplaySinglePageContinuous:
		{
			while (min < max) {
				mid = min + (max - min) / 2;
				float bottom = _pdfPriv->pageViewBounds[mid].origin.y;
				float top = bottom + _pdfPriv->pageViewBounds[mid].size.height + padding;
				
				if (point.y < bottom) {
					min = ++mid;
				} else if (point.y > top) {
					max = --mid;
				} else {
					break;
				}
			}
			result = [doc pageAtIndex:mid];
		}
			break;
		case kPDFDisplayTwoUpContinuous:
		{
			int facingIndex = 0;
			// first find y
			while (min < max) {
				mid = min + (max - min) / 2;
				PDFPage *facingPage = [self facingPageForPage:[doc pageAtIndex:mid]];
				facingIndex = facingPage != nil ? [doc indexForPage:facingPage] : mid;
				float bottom = MIN(_pdfPriv->pageViewBounds[mid].origin.y, _pdfPriv->pageViewBounds[facingIndex].origin.y);
				float top = bottom + padding + 
				MAX(_pdfPriv->pageViewBounds[mid].size.height, _pdfPriv->pageViewBounds[facingIndex].size.height); 
				
				if (point.y < bottom) {
					min = mid = mid + 2 <= lastPageIndex ? mid + 2 : lastPageIndex;
				} else if (point.y > top) {
					max = mid = mid - 2 >= 0 ? mid - 2 : 0;
				} else {
					break;
				}
			}
			
			// now find x
			PDFPage *facingPage = [self facingPageForPage:[doc pageAtIndex:mid]];
			facingIndex = facingPage != nil ? [doc indexForPage:facingPage] : mid;
			int left, right;
			if (_pdfPriv->pageViewBounds[mid].origin.x < _pdfPriv->pageViewBounds[facingIndex].origin.x) {
				left = mid;
				right = facingIndex;
			} else {
				left = facingIndex;
				right = mid;
			}
			
			mid = point.x < _pdfPriv->pageViewBounds[right].origin.x - padding ? left : right;
			result = [doc pageAtIndex:mid];
		}
			break;
		default:
			result = nil;
			break;
	} 
	return result;
}


- (NSPoint)originForPageDisplay:(PDFPage*)page
{
	NSPoint result = {0,0};
	
	PDFDocument *doc = [self document];
	
	if (page && doc) {
		if (_pdfPriv->pageViewBounds == nil) {
			[self generatePageViewBounds];
		}
		
		int index = [doc indexForPage:page];
		result = _pdfPriv->pageViewBounds[index].origin;
	}
	

	return result;
}


- (NSRect)normalizedPageBounds:(PDFPage*)page
{
	NSRect rect = {{0,0},{0,0}};

	rect = [page boundsForBox:[self displayBox]];

	return rect;
}


- (float)pageViewHeight:(PDFPage*)page;
{
	float pageViewHeight = 0;
	
	if (page != nil) {
		pageViewHeight = [self normalizedPageBounds:page].size.height;
		
		PDFDisplayMode mode = [self functionalDisplayMode];
		if (mode == kPDFDisplayTwoUp || mode == kPDFDisplayTwoUpContinuous) {
			page = [self facingPageForPage:page];
			pageViewHeight = MAX(pageViewHeight, 
								 [self normalizedPageBounds:page].size.height);
		}
		
		if ([self displaysPageBreaks]) {
			pageViewHeight +=  PageBreakPad;
		}
		
		pageViewHeight = roundf(pageViewHeight *= [self scaleFactor]);
	}
	return pageViewHeight;
}


- (BOOL)pageIsLeftOfAPair:(id)arg1
{
	return 0;
}


- (BOOL)pageIsRightOfAPair:(id)arg1
{
	return 0;
}

- (BOOL)page:(id)arg1 isNeighboringPage:(id)arg2
{
	return NO;
}



- (NSPoint)scrollOriginForPageTopLeft:(PDFPage*)page
{
	NSRect box = [page boundsForBox:[self displayBox]];
	BOOL displaysPageBreaks = [self displaysPageBreaks];
	NSPoint origin;
	
	switch ([page rotation]) {
		case 0:
			origin.x = box.origin.x;
			origin.y = box.origin.y + box.size.height;
			
			if (displaysPageBreaks) {
				origin.x -= PageBreakPad / 2.;
				origin.y += PageBreakPad / 2.;
			}
			break;
		case 90:
			origin.x = box.origin.x;
			origin.y = box.origin.y;
			
			if (displaysPageBreaks) {
				origin.x -= PageBreakPad / 2.;
				origin.y -= PageBreakPad / 2.;
			}
			break;
		case 180:
			origin.x = box.origin.x + box.size.width;
			origin.y = box.origin.y;
			
			if (displaysPageBreaks) {
				origin.x += PageBreakPad / 2.;
				origin.y -= PageBreakPad / 2.;
			}
			break;
		case 270:
			origin.x = box.origin.x + box.size.width;
			origin.y = box.origin.y + box.size.height;
			
			if (displaysPageBreaks) {
				origin.x += PageBreakPad / 2.;
				origin.y += PageBreakPad / 2.;
			}
			break;
		default:
			break;
	}
	
	
	origin = [self convertPoint:origin fromPage:page];
	origin = [_pdfPriv->matteView convertPoint:origin fromView:self];
	
	return origin;
	
}


- (void)setGutterColor:(id)arg1
{
}


-(NSColor*) gutterColor
{
	return _pdfPriv->gutterColor;
}


-(void) setGutterDefaultsToRightSide:(BOOL)val
{
	_pdfPriv->gutterOnRight = val;
}


-(BOOL) gutterDefaultsToRightSide
{	
	return _pdfPriv->gutterOnRight;
}


- (float)gutterOffsetForPageDisplay:(PDFPage*)page
{
	float result = 0;

	return result;
}

- (NSRect)gutterBoundsForPage:(id)arg1
{
	NSRect rect = {{0,0},{0,0}};
	return rect;
}

-(void) setDoPeriodicFlush:(BOOL) flush
{
}

-(BOOL) doPeriodicFlush
{	
	return _pdfPriv->periodicFlush;
}

- (void)setPeriodicFlushInterval:(CGFloat)arg1
{
}

-(float) periodicFlushInterval
{	
	return _pdfPriv->flushInterval;
}

-(BOOL) doDrawPageContent
{	
	return _pdfPriv->drawPageContent;
}

- (void)handleInitialContentRenderSkip
{
}


- (void)scheduledRedisplay:(id)arg1
{
	NSLog(@"scheduledRedisplay");
}

- (unsigned int)lastPageIndex
{
	unsigned int res = [_pdfPriv->pdfDoc pageCount];
	res = res > 0 ? res - 1 : 0;

	return res;
}

- (void)appendPasswordUI:(id)arg1
{
}

-(BOOL) passwordUIDisplayed
{
	return _pdfPriv->passwordViewUp;
}

- (void)documentWasUnlocked:(id)arg1
{
}

- (void)documentAnnotationsChanged:(id)arg1
{
}

- (void)removePasswordUI:(id)arg1
{
}


- (id)nextPage
{
	id res = nil;
	

	if ([self canGoToNextPage]) {
		res = [_pdfPriv->pdfDoc pageAtIndex:_pdfPriv->currentPageIndex + 1];
	}

	return res;
}


- (id)previousPage
{
	PDFPage *page = nil;
	

	return page;
}

- (void)goToTopOfNextPage:(id)sender
{
	NSLog(@"%s", __PRETTY_FUNCTION__);
	PDFPage *page = [self nextPage];
	if (page != nil) {
		NSPoint top = [self topOfPage:page];
		PDFDestination *dest = [[PDFDestination alloc] initWithPage:page atPoint:top];
		[self goToDestination:dest];
	}
	
}

- (void)goToBottomOfPreviousPage:(id)sender
{
	PDFPage *page = [self previousPage];
	if (page != nil) {
		NSPoint point = [self bottomOfPage:page];
		PDFDestination *dest = [[PDFDestination alloc] initWithPage:page atPoint:point];
		[self goToDestination:dest];
	}
}



- (void)pushDestination:(PDFDestination*)dest
{
	if (dest == nil) {
		return;
	}
	
	
	
	if (_pdfPriv->historyIndex < [_pdfPriv->destinationHistory count]) {
		NSRange range = NSMakeRange(_pdfPriv->historyIndex, 
									[_pdfPriv->destinationHistory count] - _pdfPriv->historyIndex);
		[_pdfPriv->destinationHistory removeObjectsInRange:range];
	}
	
	_pdfPriv->historyIndex += 1;
	[_pdfPriv->destinationHistory addObject:dest];
	
	NSNotificationQueue *queue = [NSNotificationQueue defaultQueue];
	NSNotification *notif = [NSNotification 
							 notificationWithName:PDFViewChangedHistoryNotification
							 object:self];
	
	[queue enqueueNotification:notif postingStyle:NSPostWhenIdle];
	
}




- (void)goToDestinationNoPush:(PDFDestination*)dest
{
	PDFPage *page = nil;
	if (dest && (page = [dest page])) {
		NSPoint point = [dest point];
		
		
		
		if (page == [self currentPage]) {
			// return on some cond
		}
		
		PDFDisplayMode mode = [self functionalDisplayMode];
		if (mode == kPDFDisplaySinglePage || mode == kPDFDisplayTwoUp) {
			//
			[self goToPageNoPush:page];
		}
		
		
		NSPoint documentPoint = [_pdfPriv->scrollView documentVisibleRect].origin;
		
		int rotation = [page rotation];
		
		if (rotation == 90) {
			//
		}
		
		
		if (point.x >= FLT_MAX) {
			point.x = documentPoint.x;
		}
		
		NSPoint pagePoint = [self convertPoint:point fromPage:page];
		
		NSPoint viewPoint = [_pdfPriv->matteView convertPoint:pagePoint fromView:self];
		
		NSLog(@"%s\npoint: x:%.3f, y: %.3f, pagePoint.x: %.3f, pagePoint.y: %.3f, viewPoint.x: %.3f, viewPoint.y: %.3f dr.x: %.3f, dr.y: %.3f", 
			  __PRETTY_FUNCTION__, point.x, point.y, pagePoint.x, pagePoint.y, viewPoint.x, viewPoint.y, documentPoint.x, documentPoint.y);
		
		if (rotation != 0) {
			//
		}
		

		float x = point.x == documentPoint.x ? point.x : viewPoint.x;
		
		NSPoint scrollPoint = {x,viewPoint.y};
		[self constrainedScrollToPoint:scrollPoint];
		
		[self determineCurrentPage];
		
		if ([[self document] indexForPage:page] !=
			_pdfPriv->currentPageIndex) {
			//
		}
	}
}




- (void)goToPageNoPush:(PDFPage*) page
{
	
	if (page != nil) {
		int oldPageIndex = _pdfPriv->currentPageIndex;
		_pdfPriv->currentPageIndex = [[self document] indexForPage:page];
		
		if (!_pdfPriv->inhibitAutoScroll && _pdfPriv->pagesView) {
			[self reflectNewPageOn];
		}
		
		if (oldPageIndex != _pdfPriv->currentPageIndex) {
			NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
			
			[defaultCenter postNotificationName:PDFViewPageChangedNotification object:self];
		}
	}
}

- (BOOL)isRectVisible:(NSRect)arg1 onPage:(id)arg2
{
	BOOL result = NO;

	return result;
}

-(void) slamScaleFactor:(float)scaleFactor
{	
	_pdfPriv->scaleFactor = scaleFactor;
}


-(float) maxScaleFactor
{
	return _pdfPriv->maxScaleFactor;
}


-(float) minScaleFactor
{	
	return _pdfPriv->minScaleFactor;
}


-(BOOL) inhibitDrawing
{	
	return _pdfPriv->inhibitDrawing;
}


-(void) setInhibitDrawing:(BOOL)inhibit
{	
	_pdfPriv->inhibitDrawing = inhibit;
}


- (void)drawPagePre:(PDFPage*)page
{

	NSRect bounds = [page boundsForBox:[self displayBox]];
	NSRect cnvrtBounds = [self convertRect:bounds fromPage:page];
	
	NSRect viewRect = [self convertRect:cnvrtBounds 
								 toView:[self documentView]];
	
	[[self pageColor] set];
	
	if ([self displaysPageBreaks]) {
		[self drawShadowedRect:viewRect blur:4.0];
	}
	
	if (_pdfPriv->pageColorHasAlpha) {
		NSRectFillUsingOperation(viewRect, NSCompositeSourceOver);
		
	} else {
		NSRectFill(viewRect);
	}
}


- (void)drawSelection:(id)arg1 forPage:(id)arg2
{
}


- (void)drawCPSelection:(id)arg1 forPage:(id)arg2
{
}


- (void)drawShadowedRect:(NSRect) rect blur:(float)blur
{
}


- (void)createShadowMaskParts
{
}

-(BOOL) backgroundHasAlpha
{	
	return _pdfPriv->matteColorHasAlpha;
}


- (void)setAnnotationNeedsDisplay:(id)arg1 padding:(CGFloat)arg2
{
}



- (void)initInternalViews
{

	_pdfPriv->pagesView = [[[MyPDFDisplayView alloc] initWithFrame:[self bounds]] autorelease];
	[_pdfPriv->pagesView setParentPDFView:self];
	
	_pdfPriv->matteView = [[[MyPDFMatteView alloc] initWithFrame:[self bounds]] autorelease];
	[_pdfPriv->matteView addSubview:_pdfPriv->pagesView];
	
	_pdfPriv->scrollView = [[[NSScrollView alloc] initWithFrame:[self bounds]] autorelease];
	[_pdfPriv->scrollView setDocumentView:_pdfPriv->matteView];
	[_pdfPriv->scrollView setHasVerticalScroller:NO];
	[_pdfPriv->scrollView setHasHorizontalScroller:NO];
	[[_pdfPriv->scrollView contentView] setCopiesOnScroll:YES];
	
	if (_pdfPriv->bezeled) {
		//
	}
	
	[[_pdfPriv->scrollView contentView] _setCanAnimateScrolls:NO];
	[self addSubview:_pdfPriv->scrollView];
	[self setAutoresizesSubviews:NO];
	
	NSNotificationCenter *defaultCenter = [NSNotificationCenter defaultCenter];
	[defaultCenter addObserver:self selector:@selector(selfDidResize:) 
						  name:NSViewFrameDidChangeNotification 
						object:self];
	
	[defaultCenter addObserver:self selector:@selector(clipViewBoundsChanged:) 
						  name:NSViewBoundsDidChangeNotification 
						object:[_pdfPriv->scrollView contentView]];
	
	[defaultCenter addObserver:self selector:@selector(visiblePagesChanged:) 
						  name:@"PDFViewVisiblePagesChanged" 
						object:self];
	
	[_pdfPriv->matteView setPDFView:self];
	
	[self performSelector:@selector(setupWindowNotifications:) withObject:nil afterDelay:0.0];
	
}


- (void)setupWindowNotifications:(id)sender
{
	NSWindow *window = [self window];
	NSNotificationCenter *dc = [NSNotificationCenter defaultCenter];
	
	[dc addObserver:self selector:@selector(windowDidChangeKeyState:) 
			   name:NSWindowDidBecomeKeyNotification object:window];
	
	[dc addObserver:self selector:@selector(windowDidChangeKeyState:)
			   name:NSWindowDidResignKeyNotification object:window];
	
	[window setAcceptsMouseMovedEvents:YES];
	
	_pdfPriv->windowNotifications = YES;
}



- (void)selfDidResize:(id)sender
{
	
	if (_pdfPriv->pagesView != nil) {
		
		NSRect bounds = [self bounds];
		[_pdfPriv->scrollView setFrame:bounds];
		
		if ([self autoScales]) {
			[self resizeDisplayView:nil];
		} else	{
			[self positionInternalViews:nil];
		}
		
		if ([[self window] isVisible] == NO) {
			PDFDisplayMode mode = [self functionalDisplayMode];
			if (mode == kPDFDisplaySinglePage || mode == kPDFDisplayTwoUp) {
				[self reflectNewPageOn];
			}
		}
		
		[self syncPageIndexToScrollView];
	}
	
}

- (void)clipViewBoundsChanged:(id)sender
{
	[self syncPageIndexToScrollView];
}


- (void)visiblePagesChanged:(id)sender
{
}


- (void)windowDidChangeKeyState:(id)arg1
{
}

- (void)reflectNewPageOn
{
		
	PDFDisplayMode mode = [self functionalDisplayMode];
	
	if (mode == kPDFDisplaySinglePage || mode == kPDFDisplayTwoUp) {
		[self freePageViewBounds];
		NSSize zeroSize = {0,0};
		_pdfPriv->singlePageContSize = zeroSize;
		_pdfPriv->twoUpContSize = zeroSize;
		[self resizeDisplayView:self];
		[self setNeedsDisplay:YES];
		
		
	} else {
		NSPoint origin = [self scrollOriginForPageTopLeft:[self currentPage]];
		[self constrainedScrollToPoint:origin];
	}
	
	_pdfPriv->scrollEntry = 0;
	
}

- (void)scheduleSyncPageIndex:(id)sender
{
	NSRect bounds = [self bounds];
	_pdfPriv->scheduledPageSync = NO;
	
	bounds.origin.y += bounds.size.height / 2.;
	PDFPage *page = [self nearestPageToPoint:bounds.origin];
	_pdfPriv->inhibitAutoScroll = YES;
	[self goToPageNoPush:page];
	_pdfPriv->inhibitAutoScroll = NO;
}


- (void)cancelScheduleSyncPageIndex
{
	[NSObject cancelPreviousPerformRequestsWithTarget:self 
											 selector:@selector(scheduleSyncPageIndex:)
											object:nil];
	_pdfPriv->scheduledPageSync = NO;
}

- (void)syncPageIndexToScrollView
{
	PDFDisplayMode mode = [self functionalDisplayMode];
	if ([self document] != nil &&
		(mode == kPDFDisplaySinglePageContinuous || mode == kPDFDisplayTwoUpContinuous) &&
		_pdfPriv->scheduledPageSync == NO) {
		[self performSelector:@selector(scheduleSyncPageIndex:)
				   withObject:nil
				   afterDelay:[self scheduleSyncPageDelay]];
		_pdfPriv->scheduledPageSync = YES;
	}
}


- (PDFPage*)determineCurrentPage
{
	PDFPage *result = nil;
	NSRect bounds = [self bounds];
	

	PDFDisplayMode dispMode = [self functionalDisplayMode];
	
	if (dispMode == kPDFDisplaySinglePage || dispMode == kPDFDisplayTwoUp) {
		result = [self currentPage];
	} else {
		
		
		NSPoint point = {0, bounds.size.height / 2.};
		
		result = [self pageForPoint:point nearest:YES];
	}
	

	return result;
}



- (NSSize)pageViewSizeForPage:(PDFPage*)page
{

	PDFDisplayMode mode = [self functionalDisplayMode];
	NSSize size = [self viewSizeForPage:page];
	size.width = roundf(size.width * _pdfPriv->scaleFactor);
	size.height = roundf(size.height * _pdfPriv->scaleFactor);
	
	if (_pdfPriv->autoScale && (mode == kPDFDisplaySinglePage || mode == kPDFDisplayTwoUp)) {
		NSSize frameSize = [_pdfPriv->scrollView documentVisibleRect].size;
		size.height = MIN(size.height, frameSize.height);
		size.width = MIN(size.width, frameSize.width);
	}

	return size;
}

- (void)resizeDisplayView:(id)sender
{

	NSPoint point = [_pdfPriv->scrollView documentVisibleRect].origin;
	
	PDFDisplayMode dispMode = [self functionalDisplayMode];
	
	float currentScaleFactor = _pdfPriv->scaleFactor;
	
	if (dispMode == kPDFDisplaySinglePageContinuous || dispMode == kPDFDisplayTwoUpContinuous) {
		point.x /= currentScaleFactor;
		point.y /= currentScaleFactor;
	}
	
	if ([self autoScales]) {
		_pdfPriv->scaleFactor = [self autoScaleFactor];
	}
	
	PDFPage *page = [self currentPage];
	NSSize size = [self pageViewSizeForPage:page];
	[_pdfPriv->pagesView setFrameSize:size];
	
	[self positionInternalViews:nil];
	
	[self layoutPopupViews];
	
	if (dispMode == kPDFDisplaySinglePageContinuous || dispMode == kPDFDisplayTwoUpContinuous) {
		point.x *= _pdfPriv->scaleFactor;
		point.y *= _pdfPriv->scaleFactor;
		
		[self constrainedScrollToPoint:point];
	}
	
	[self resetPDFTooltips];
	
	if (_pdfPriv->scaleFactor != currentScaleFactor) {
		NSNotificationCenter *dc = [NSNotificationCenter defaultCenter];
		[dc postNotificationName:PDFViewScaleChangedNotification object:self];
	}
}




- (void)positionInternalViews:(id)arg1
{
	NSSize scrollSize = [_pdfPriv->scrollView frame].size;
	NSSize pagesSize = [_pdfPriv->pagesView frame].size;
	
	NSSize newMatteSize;
	NSPoint newPagesOrigin;
	
	
	if ([self autoScales]) {
		if ([_pdfPriv->scrollView hasHorizontalScroller]) {
			[_pdfPriv->scrollView setHasVerticalScroller:NO];
		}
		if (pagesSize.height > scrollSize.height) {
			if ([_pdfPriv->scrollView hasVerticalScroller] == NO) {
				[_pdfPriv->scrollView setHasVerticalScroller:YES];
			}
		} else {
			if ([_pdfPriv->scrollView hasVerticalScroller] == YES) {
				[_pdfPriv->scrollView setHasVerticalScroller:NO];
			}
		}
	} else {
		if (scrollSize.width >= pagesSize.width) {
			if (scrollSize.height >= pagesSize.height) {
				// scroll.height >=  pages.height && 
				// scroll.width  >=  pages.width
				if ([_pdfPriv->scrollView hasVerticalScroller]) {
					[_pdfPriv->scrollView setHasVerticalScroller:NO];
				}
			} else {
				// scroll.height <   pages.height && 
				// scroll.width  >=  pages.width
				
				if ([_pdfPriv->scrollView hasVerticalScroller] == NO) {
					[_pdfPriv->scrollView setHasVerticalScroller:YES];
				}
				
				// scroll bar turned on, make sure still size for horz scroller
				float scrollerWidth = [NSScroller scrollerWidthForControlSize:
									   [[_pdfPriv->scrollView verticalScroller] controlSize]];
				
				if (pagesSize.width > (scrollSize.width - scrollerWidth)) {
					if ([_pdfPriv->scrollView hasHorizontalScroller] == NO) {
						[_pdfPriv->scrollView setHasHorizontalScroller:YES];
					}
				}
			}
			
			// disable horz scroll bar if it exists
			if ([_pdfPriv->scrollView hasHorizontalScroller]) {
				[_pdfPriv->scrollView setHasHorizontalScroller:NO];
			}
		} else {
			if(scrollSize.height >= pagesSize.height) {
				// scroll.width  <   pages.width &&
				// scroll.height >=  pages.height && 
				if ([_pdfPriv->scrollView hasHorizontalScroller] == NO) {
					[_pdfPriv->scrollView setHasHorizontalScroller:YES];
				}
				
				if ([_pdfPriv->scrollView hasVerticalScroller]) {
					[_pdfPriv->scrollView setHasVerticalScroller:NO];
				}
				
			} else {
				// scroll.width  <   pages.width &&
				// scroll.height <   pages.height && 
				if ([_pdfPriv->scrollView hasHorizontalScroller] == NO) {
					[_pdfPriv->scrollView setHasHorizontalScroller:YES];
				}
				
				if ([_pdfPriv->scrollView hasVerticalScroller] == NO) {
					[_pdfPriv->scrollView setHasVerticalScroller:YES];
				}
			}
		}
	}
	
	NSSize documentSize = [_pdfPriv->scrollView documentVisibleRect].size;
	
	if(pagesSize.width > documentSize.width) {
		newMatteSize.width = pagesSize.width;
		newPagesOrigin.x = 0;
	} else {
		newMatteSize.width = documentSize.width;
		newPagesOrigin.x = round((documentSize.width - pagesSize.width)/2.0);
		
	}
	
	if (pagesSize.height > documentSize.height) {
		newMatteSize.height = pagesSize.height;
		newPagesOrigin.y = 0;
	} else {
		newMatteSize.height = documentSize.height;
		newPagesOrigin.y = round((documentSize.height - pagesSize.height)/2.0);
	}
	
	[_pdfPriv->matteView setFrameSize:newMatteSize];
	[_pdfPriv->pagesView setFrameOrigin:newPagesOrigin];
	

}


- (void)constrainedScrollToPoint:(NSPoint)point
{
	NSClipView *contentView = [_pdfPriv->scrollView contentView];
	NSPoint constrainedPoint = [contentView constrainScrollPoint:point];
	constrainedPoint.x = roundf(constrainedPoint.x);
	constrainedPoint.y = roundf(constrainedPoint.y);
	[contentView scrollToPoint:constrainedPoint];
	[_pdfPriv->scrollView reflectScrolledClipView:contentView];
}




- (void)scrollVerticalBy:(float)scroll;
{

	
	PDFPage *currentPage = [self determineCurrentPage];
	
	NSRect currentRect = [_pdfPriv->scrollView documentVisibleRect];
	NSPoint newOrigin = {currentRect.origin.x, roundf(currentRect.origin.y + scroll)};
	
	if ([self shouldAnimateScroll]) {
		NSLog(@"should animate scroll not implemented yet");
	} else {
		[self constrainedScrollToPoint:newOrigin];
		
		PDFDisplayMode dispMode = [self functionalDisplayMode];
		
		if (dispMode == kPDFDisplaySinglePage || dispMode == kPDFDisplayTwoUp) {
			NSRect newRect = [_pdfPriv->scrollView documentVisibleRect];
			
			//did it move?
			if (newRect.origin.y == currentRect.origin.y) {
				if (scroll > 0) {
					// scroll forward
					if ([self canGoToNextPage]) {
						newOrigin.y = 0;
						[self constrainedScrollToPoint:newOrigin];
						[self goToNextPage:self];
					} 
					
				} else {
					// scroll backwards
					if ([self canGoToPreviousPage]) {
						newOrigin.y = newRect.size.height;
						[self constrainedScrollToPoint:newOrigin];
						[self goToPreviousPage:self];
					}
				}
			}
			
		} else {
			PDFPage *newPage = [self determineCurrentPage];
			if (newPage != currentPage) {
				_pdfPriv->inhibitAutoScroll = YES;
				[self goToPage:newPage];
				_pdfPriv->inhibitAutoScroll = NO;
			}
		}
	}
}

- (void)scrollHorizontalBy:(CGFloat)arg1
{
}


- (void)scrollByPage:(BOOL)arg1
{
}




- (NSPoint)topOfPage:(PDFPage*)page
{
	int box = [self displayBox];
	NSRect bounds = [page boundsForBox:box];
		
	NSPoint point;
	point.x = FLT_MAX;
	point.y = bounds.size.height;
	
	return point;
}


- (NSPoint)bottomOfPage:(PDFPage*)page
{
	int box = [self displayBox];
	NSPoint bottom = [page boundsForBox:box].origin;
	
	
	bottom.x = FLT_MAX;
	
	return bottom;
}



- (void)findVisiblePages
{

	
	PDFDisplayMode displayMode = [self functionalDisplayMode];
	PDFDocument *doc = [self document];
	unsigned int newFirstVisiblePage = 0;
	unsigned int newLastVisiblePage = 0;
	
	switch (displayMode) {
		case kPDFDisplaySinglePage:
			newFirstVisiblePage = _pdfPriv->currentPageIndex + 1;
			newLastVisiblePage = _pdfPriv->currentPageIndex + 1;
			break;
		case kPDFDisplaySinglePageContinuous: {
			NSRect bounds = [self bounds];
			PDFPage *lastPage = [self pageForPoint:bounds.origin nearest:YES];
			bounds.origin.x += bounds.size.width;
			bounds.origin.y += bounds.size.height;
			PDFPage *firstPage = [self pageForPoint: bounds.origin nearest:YES];
			newFirstVisiblePage = [doc indexForPage:firstPage] + 1;
			newLastVisiblePage = [doc indexForPage:lastPage] + 1;
		}
			break;
		case kPDFDisplayTwoUp: {
			PDFPage *facingPage = [self facingPageForPage:
								   [doc pageAtIndex:_pdfPriv->currentPageIndex]];
			int facingPageIndex = facingPage != nil ? 
			[doc indexForPage:facingPage] : _pdfPriv->currentPageIndex;
			newFirstVisiblePage = MIN(_pdfPriv->currentPageIndex, facingPageIndex) + 1;
			newLastVisiblePage = MAX(_pdfPriv->currentPageIndex, facingPageIndex) + 1;
		}
			break;
		case kPDFDisplayTwoUpContinuous: {
			NSRect bounds = [self bounds];
			PDFPage *lastPage = [self pageForPoint:bounds.origin nearest:YES];
			PDFPage *lastFacingPage = [self facingPageForPage:lastPage];
			bounds.origin.x += bounds.size.width;
			bounds.origin.y += bounds.size.height;
			PDFPage *firstPage = [self pageForPoint: bounds.origin nearest:YES];
			PDFPage *firstFacingPage = [self facingPageForPage:firstPage];
			
			newFirstVisiblePage = firstFacingPage != nil ? 
			MIN([doc indexForPage:firstPage], [doc indexForPage:firstFacingPage]) + 1 :
			[doc indexForPage:firstPage] + 1;
			
			newLastVisiblePage = lastFacingPage != nil ?
			MAX([doc indexForPage:lastPage], [doc indexForPage:lastFacingPage]) + 1 :
			[doc indexForPage:lastPage] + 1;
			
		}
			break;	
		default:
			break;
	}
	
	
	
	if (newFirstVisiblePage != _pdfPriv->firstVisiblePage ||
		newLastVisiblePage != _pdfPriv->lastVisiblePage) {
		
		NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
								  [NSNumber numberWithUnsignedInteger:_pdfPriv->firstVisiblePage - 1],	PDFViewWasFirstVisiblePage,
								  [NSNumber numberWithUnsignedInteger:_pdfPriv->lastVisiblePage - 1],	PDFViewWasLastVisiblePage,
								  [NSNumber numberWithUnsignedInteger:newFirstVisiblePage - 1],		PDFViewFirstVisiblePage,		
								  [NSNumber numberWithUnsignedInteger:newLastVisiblePage - 1],		PDFViewLastVisiblePage,		
								  nil];
		
		NSNotification *notification = [NSNotification notificationWithName:PDFViewVisiblePagesChangedNotification 
																	 object:self
																   userInfo:userInfo];
		
		[[NSNotificationQueue defaultQueue] enqueueNotification:notification postingStyle:NSPostASAP];
		
		_pdfPriv->firstVisiblePage = newFirstVisiblePage;
		_pdfPriv->lastVisiblePage = newLastVisiblePage;
	}
}

- (void)setPopupsAreEditable:(BOOL)arg1
{
}

-(BOOL) popupsAreEditable
{	
	return _pdfPriv->popupsEditable;
}

- (void)scanForPopupsUserOpened:(BOOL)arg1
{
}

- (void)layoutPopupViews 
{
}

- (void)removeAllPopupViews
{
}

- (void)addPopupView:(id)arg1
{
}

- (void)removePopupView:(id)arg1
{
}

- (void)movePopupToFrontForAnnotation:(id)arg1
{
}

- (void)hitTestPopupViews:(id)arg1
{
}

- (NSRect)popupFrameForAnnotation:(id)arg1
{
	NSRect rect = {{0,0},{0,0}};
	return rect;
}

- (NSRect)popupFrameForAnnotation:(id)arg1 gutterBounds:(NSRect)arg2
{
	NSRect rect = {{0,0},{0,0}};
	return rect;
}

- (id)popupViewForAnnotation:(id)arg1
{
	return nil;
}

- (void)positionPopups:(id)arg1 inRect:(NSRect)arg2
{
}

- (float)movePopups:(id)arg1 atAndBeforeIndex:(unsigned int)arg2 upBy:(float)arg3 gutterBounds:(NSRect)arg4 lockedBubbleIndex:(unsigned int *)arg5
{
	return 0;
}

- (void)popupViewMoveToFront:(id)arg1
{
}

- (void)popupViewClose:(id)arg1
{
}


- (void)addTooltipsForVisiblePages
{
}

- (void)addTooltipsForPage:(id)arg1
{
}

- (id)view:(id)arg1 stringForToolTip:(long)arg2 point:(NSPoint)arg3 userData:(void *)arg4
{
	return nil;
}

- (void)scrollToSelection:(id)arg1
{
}

- (void)invalidateSelectionBounds:(id)arg1
{	
}

- (BOOL)selectedTextActive
{
	return NO;
}

- (void)internalClearSelection
{
}

- (void)animateTransitionFromPage:(id)arg1 toPage:(id)arg2
{
}

- (void)endPageTransition:(id)arg1 context:(void *)arg2
{
}

-(unsigned int) scrollEntryPoint
{	
	return _pdfPriv->scrollEntry;
}


- (BOOL)shouldAnimateScroll
{
	return NO;
}

-(BOOL) animateNeighboringPageChangeOnly
{
	return NO;
}

-(float) maximumScrollAnimationDistance
{ 
	return 3000.;
}

- (void)setAnimateFlag:(unsigned int)arg1
{
}

-(unsigned int) animateFlag
{
	return _pdfPriv->animateFlag;
}

- (void)startAnimateZoomToScaleFactor:(CGFloat)arg1
{
}

-(BOOL) suspendRendering
{	
	return _pdfPriv->suspendRendering;
}

- (void)createTransition:(int)arg1 forView:(id)arg2
{
}

- (void)createCurrentImageFromView:(id)arg1
{
	NSLog(@"%s", __PRETTY_FUNCTION__);
}

- (void)handleAnimatePageTransition:(id)arg1
{
}

- (void)handleAnimateScrollFromPt:(NSPoint)arg1 toPt:(NSPoint)arg2
{
}

- (void)handleAnimationProgress
{
}

- (void)animationDidEnd:(NSAnimation *)animation
{
}

-(float) animationDuration
{
	return 0.3;
}

- (void)stopPDFViewAnimation
{
}

- (void)resetPDFTooltips
{
}

- (void)runDataDetectorsOnVisiblePages
{
}


- (float)autoScaleFactorForPage:(PDFPage*)page 
{
	float scale = 0;

	
	NSSize pageSize = [self viewSizeForPage:page];
	NSSize bounds = [self bounds].size;
	PDFDisplayMode mode = [self functionalDisplayMode];
	
	if (mode == kPDFDisplaySinglePage || mode == kPDFDisplayTwoUp) {
		float xscale = bounds.width / pageSize.width;
		float yscale = bounds.height / pageSize.height;
		
		scale =  (xscale * pageSize.height > bounds.height) ? yscale : xscale;
	} else {
		float scrollerSize = [NSScroller scrollerWidthForControlSize:
							  [[_pdfPriv->scrollView verticalScroller] controlSize]];
		scale = (bounds.width - scrollerSize) / pageSize.width;
	}
	
	
	
	id delagate = [self delegate];
	if ([delagate respondsToSelector:@selector(PDFViewWillChangeScaleFactor:toScale:)] == NO) {
		delagate = self;
	}
	
	scale = [delagate PDFViewWillChangeScaleFactor:self toScale:scale];
	
	
	return scale;
}





@end
