#import "MyPDFView.h"
#import "MyPDFViewPrivate.h"
#import "MyPDFViewInternal.h";


@implementation MyPDFView (MyPDFViewPrivate)

- (float)autoScaleFactor
{
	return [self autoScaleFactorForPage:[self currentPage]];
}


- (void)setScaleFactor:(float)scaleFactor anchorPoint:(NSPoint)point
{
	id delegate = [self delegate];
	if (delegate == nil || [delegate respondsToSelector:@selector(PDFViewWillChangeScaleFactor:toScale:)] == NO) {
		delegate = self;
	}
	
	scaleFactor = [delegate PDFViewWillChangeScaleFactor:self toScale:scaleFactor];
	
	[self setAutoScales:NO];
	
	if (_pdfPriv->scaleFactor != scaleFactor) {
		// save old point and rect
		PDFPage *page = [self nearestPageToPoint:point];
		NSPoint origin = [_pdfPriv->scrollView documentVisibleRect].origin;
		
		NSPoint pagePoint = [self convertPoint:point toPage:page];
		
		// set the scale, and re-layout
		_pdfPriv->scaleFactor = scaleFactor;
		[self layoutDocumentView];
		
		// calculate new scroll origin and scroll
		NSPoint newPoint = [self convertPoint:pagePoint fromPage:page];
		
		origin.y = roundf(origin.y - newPoint.y + point.y);
		origin.x = roundf(origin.x + newPoint.x - point.x);
		
		
		[self constrainedScrollToPoint:origin];
		
		NSNotificationCenter *dc = [NSNotificationCenter defaultCenter];
		[dc postNotificationName:PDFViewScaleChangedNotification object:self];
	}
}

- (id)history
{
	return [NSArray arrayWithArray:_pdfPriv->destinationHistory];
}


-(int) currentHistoryIndex
{	
	return _pdfPriv->historyIndex;
}

- (void)setUsesPageLabels:(BOOL)arg1
{
}


-(BOOL) usesPageLabels
{	
	return _pdfPriv->useLogicalPageLabels;
}


- (void)setDisplaysTooltips:(BOOL)arg1
{
}


-(BOOL) displaysTooltips
{	
	return _pdfPriv->showTooltips;
}



- (PDFPage*)facingPageForPage:(PDFPage*)page
{
	PDFPage *result = nil;
	PDFDocument *document = [self document];
	PDFDisplayMode mode = [self functionalDisplayMode];
	int lastPageIndex = [self lastPageIndex];
	int index;
	int facingIndex = -1;
	
	if (page && 
		(mode == kPDFDisplayTwoUp || mode == kPDFDisplayTwoUpContinuous) &&
		(index = [document indexForPage:page]) >= 0) {
		if ([self displaysAsBook]) {
			if ((index % 2) != 0 && index < lastPageIndex) {
				// even page -- left column
				facingIndex = index + 1;
			} else if ((index % 2) == 0 && index > 1) {
				facingIndex = index - 1;
			}
		} else {
			if ((index % 2) == 0 && index < lastPageIndex) {
				// even page -- left column
				facingIndex = index + 1;
			} else if ((index % 2) != 0 && index > 0) {
				facingIndex = index - 1;
			}
		}
		
		if (facingIndex >= 0) {
			result = [document pageAtIndex:facingIndex];
		}
	}
	return result;
}


- (void)setBezeled:(BOOL)arg1
{
}

-(BOOL) isBezeled
{
	return _pdfPriv->bezeled;
}

- (void)setPageColor:(id)arg1
{
}

-(NSColor*) pageColor
{	
	return _pdfPriv->pageColor;
}

- (void)setInterpolationQuality:(int)arg1
{
}

-(int) interpolationQuality
{	
	return _pdfPriv->interpolation;
}

-(void) setLineWidthThreshold:(float)threshold
{	
	_pdfPriv->lineWidthThreshold = threshold;
}


-(float) lineWidthThreshold
{	
	return _pdfPriv->lineWidthThreshold;
}


- (id)beginEditingAnnotation:(id)arg1
{
	return nil;
}

- (void)endEditingAnnotation
{
}


- (void)setGutterWidth:(float)arg1
{
}

-(float) gutterWidth
{	
	return _pdfPriv->gutterWide;
}

-(float) defaultGutterWidth
{	
	return _pdfPriv->defaultGutterWidth;
}

-(BOOL) automaticallyHandleGutter
{
	return YES;
}

- (void)setDisplaysMarkupAnnotations:(BOOL)arg1
{
}

-(BOOL) displaysMarkupAnnotations
{	
	return _pdfPriv->displaysMarkups;
}

- (id)textSelectCursor
{
	return nil;
}

-(float) scheduleSyncPageDelay
{
	return 0.25;
}


@end

